@@ -91,8 +91,8 @@ extern struct SyntaxDef      PopupFuncSyntax;
 #define FUNC_ID_START           F_NOP   /* 0 */
 #define FUNC_ID_END           	F_FUNCTIONS_NUM
 
-extern struct TermDef FuncTerms[F_FUNCTIONS_NUM + 1];
-extern struct SyntaxDef FuncSyntax;
+extern struct SyntaxDef *pFuncSyntax;
+extern struct SyntaxDef DummyFuncSyntax;
 
 /* used in some "special" function to correctly process trailing function definition */
 unsigned long TrailingFuncSpecial (struct ConfigDef * config, struct FreeStorageElem ** storage, int skip_tokens);
@@ -209,6 +209,11 @@ struct FunctionData     *String2Func ( const char *string, struct FunctionData *
 
 #define CONFIG_ID_END					(CONFIG_SUBOPTIONS_IDS+23)
 
+
+/* must call this one to fix all the pointers referencing libAfterStep */
+void LinkAfterStepConfig();
+
+
 /***************************************************************************/
 /*                        Base file pasring definitions                    */
 /***************************************************************************/
@@ -222,6 +227,7 @@ struct FunctionData     *String2Func ( const char *string, struct FunctionData *
 #define BASE_MYNAME_PATH_ID     BASE_ID_START+6
 #define BASE_DESKTOP_SIZE_ID	BASE_ID_START+7
 #define BASE_DESKTOP_SCALE_ID	BASE_ID_START+8
+#define BASE_TermCommand_ID		BASE_ID_START+9
 #define BASE_ID_END             BASE_ID_START+10
 
 typedef struct
@@ -236,6 +242,9 @@ typedef struct
     ASGeometry desktop_size;
     int desktop_scale;
 
+#define MAX_TERM_COMMANDS	16
+	char *term_command[MAX_TERM_COMMANDS] ; 
+
     struct FreeStorageElem *more_stuff;
 }BaseConfig;
 
@@ -411,9 +420,12 @@ struct FreeStorageElem **MyStyleDefs2FreeStorage (struct SyntaxDef * syntax, str
 #define MYFRAME_TitleUSat_ID			(MYFRAME_ID_START+44)
 #define MYFRAME_TitleSSat_ID			(MYFRAME_ID_START+45)
 #define MYFRAME_Inherit_ID              (MYFRAME_ID_START+46)
-#define MYFRAME_DONE_ID                 (MYFRAME_ID_START+47)
+#define MYFRAME_InheritDefaults_ID      (MYFRAME_ID_START+47)
+#define MYFRAME_TitleHSpacing_ID 		(MYFRAME_ID_START+48)
+#define MYFRAME_TitleVSpacing_ID 		(MYFRAME_ID_START+49)
+#define MYFRAME_DONE_ID                 (MYFRAME_ID_START+50)
 
-#define MYFRAME_TitleBackground_ID_START	(MYFRAME_ID_START+48)
+#define MYFRAME_TitleBackground_ID_START	(MYFRAME_DONE_ID+1)
 #define MYFRAME_LeftBtnBackground_ID		(MYFRAME_TitleBackground_ID_START+MYFRAME_TITLE_BACK_LBTN	)
 #define MYFRAME_LeftSpacerBackground_ID 	(MYFRAME_TitleBackground_ID_START+MYFRAME_TITLE_BACK_LSPACER)
 #define MYFRAME_LTitleSpacerBackground_ID 	(MYFRAME_TitleBackground_ID_START+MYFRAME_TITLE_BACK_LTITLE_SPACER)
@@ -470,6 +482,7 @@ struct FreeStorageElem **MyStyleDefs2FreeStorage (struct SyntaxDef * syntax, str
 #define BEVEL_Bottom_ID     (BEVEL_ID_START+5)
 #define BEVEL_Extra_ID      (BEVEL_ID_START+6)
 #define BEVEL_NoOutline_ID  (BEVEL_ID_START+7)
+#define BEVEL_NoInline_ID   (BEVEL_ID_START+8)
 #define BEVEL_ID_END        (BEVEL_ID_START+10)
 
 #define TBAR_LAYOUT_ID_START  		(BEVEL_ID_END+1)
@@ -569,6 +582,7 @@ typedef struct MyFrameDefinition
     int          title_fcm, title_ucm, title_scm;
 	char 		*title_fhue, *title_uhue, *title_shue; 
 	int          title_fsat, title_usat, title_ssat;
+	int          title_h_spacing, title_v_spacing;
     ASFlagType   condense_titlebar ;
 	unsigned long left_layout, right_layout ;
 	ASFlagType   left_btn_align, right_btn_align ;
@@ -734,6 +748,7 @@ typedef struct
   {
     int    rows, columns;
     ASGeometry geometry, icon_geometry;
+	int ndesks ;
     char **labels;
     char **styles;
     int    align;
@@ -745,6 +760,9 @@ typedef struct
     char *selection_color;
     char *grid_color;
     char *border_color;
+
+	int h_spacing, v_spacing ;
+
     balloonConfig *balloon_conf;
     MyStyleDefinition *style_defs;
     struct FreeStorageElem *more_stuff;
@@ -759,7 +777,6 @@ typedef struct
 #define DESK_STYLES     2
     struct MyStyle *MSDeskTitle[DESK_STYLES];
     struct MyStyle **MSDeskBack;
-    struct button_t *shade_btn ;
 
     ASFlagType  active_desk_bevel ;
     ASFlagType  inactive_desk_bevel ;
@@ -1067,6 +1084,8 @@ WinListConfig *ParseWinListOptions (const char *filename, char *myname);
 #define DATABASE_NoFocusOnMap_ID           (DATABASE_ID_START+58)
 #define DATABASE_LongLiving_ID             (DATABASE_ID_START+59)
 #define DATABASE_ShortLiving_ID            (DATABASE_ID_START+60)
+#define DATABASE_IgnoreConfig_ID           (DATABASE_ID_START+61)
+#define DATABASE_HonorConfig_ID            (DATABASE_ID_START+62)
 
 #define DATABASE_ID_END             (DATABASE_ID_START+64)
 
@@ -1160,6 +1179,7 @@ typedef struct WharfButtonContent
 {
 	char **icon;
   	struct FunctionData *function;
+	Bool unavailable ;
 }WharfButtonContent ;
 
 typedef struct WharfButton
@@ -1366,41 +1386,45 @@ int WriteWharfOptions (const char *filename, char *myname,
 #define LOOK_StartMenuSortMode_ID				(LOOK_SUPPORTED_ID_START+32)
 #define LOOK_DrawMenuBorders_ID					(LOOK_SUPPORTED_ID_START+33)
 #define LOOK_ButtonSize_ID						(LOOK_SUPPORTED_ID_START+34)			  
-#define LOOK_SeparateButtonTitle_ID				(LOOK_SUPPORTED_ID_START+35)
-#define LOOK_RubberBand_ID						(LOOK_SUPPORTED_ID_START+36)					  
-#define LOOK_WindowStyle_ID_START				(LOOK_SUPPORTED_ID_START+37)
+#define LOOK_ButtonIconSpacing_ID			 	(LOOK_SUPPORTED_ID_START+35)			  
+#define LOOK_ButtonBevel_ID					 	(LOOK_SUPPORTED_ID_START+36)			  
+#define LOOK_ButtonAlign_ID					 	(LOOK_SUPPORTED_ID_START+37)			  
+#define LOOK_SeparateButtonTitle_ID				(LOOK_SUPPORTED_ID_START+38)
+#define LOOK_RubberBand_ID						(LOOK_SUPPORTED_ID_START+39)					  
+#define LOOK_WindowStyle_ID_START				(LOOK_SUPPORTED_ID_START+40)
 #define LOOK_DefaultStyle_ID					LOOK_WindowStyle_ID_START						  
 #define LOOK_FWindowStyle_ID					(LOOK_WindowStyle_ID_START+1)						  
 #define LOOK_UWindowStyle_ID					(LOOK_WindowStyle_ID_START+2)						  
 #define LOOK_SWindowStyle_ID					(LOOK_WindowStyle_ID_START+3)						  
-#define LOOK_WindowStyle_ID_END					(LOOK_SUPPORTED_ID_START+41)
-#define LOOK_MenuStyle_ID_START					(LOOK_SUPPORTED_ID_START+41)
-#define LOOK_MenuItemStyle_ID					(LOOK_SUPPORTED_ID_START+41)					  
-#define LOOK_MenuTitleStyle_ID					(LOOK_SUPPORTED_ID_START+42)					  
-#define LOOK_MenuHiliteStyle_ID					(LOOK_SUPPORTED_ID_START+43)					  
-#define LOOK_MenuStippleStyle_ID				(LOOK_SUPPORTED_ID_START+44)					  
-#define LOOK_MenuSubItemStyle_ID				(LOOK_SUPPORTED_ID_START+45)					  
-#define LOOK_MenuHiTitleStyle_ID				(LOOK_SUPPORTED_ID_START+46)
-#define LOOK_MenuStyle_ID_END					(LOOK_SUPPORTED_ID_START+47)		  
-#define LOOK_MenuItemCompositionMethod_ID		(LOOK_SUPPORTED_ID_START+47)	  
-#define LOOK_MenuHiliteCompositionMethod_ID		(LOOK_SUPPORTED_ID_START+48)
-#define LOOK_MenuStippleCompositionMethod_ID	(LOOK_SUPPORTED_ID_START+49)
-#define LOOK_ShadeAnimationSteps_ID				(LOOK_SUPPORTED_ID_START+50)	  
-#define LOOK_TitleButtonBalloonBorderHilite_ID	(LOOK_SUPPORTED_ID_START+51)
-#define LOOK_TitleButtonBalloonXOffset_ID		(LOOK_SUPPORTED_ID_START+52)
-#define LOOK_TitleButtonBalloonYOffset_ID		(LOOK_SUPPORTED_ID_START+53)
-#define LOOK_TitleButtonBalloonDelay_ID			(LOOK_SUPPORTED_ID_START+54)		  
-#define LOOK_TitleButtonBalloonCloseDelay_ID	(LOOK_SUPPORTED_ID_START+55)
-#define LOOK_TitleButtonBalloonStyle_ID			(LOOK_SUPPORTED_ID_START+56)
-#define LOOK_TitleButtonBalloons_ID				(LOOK_SUPPORTED_ID_START+57)
-#define LOOK_TitleButton_ID						(LOOK_SUPPORTED_ID_START+58)
-#define LOOK_KillBackgroundThreshold_ID			(LOOK_SUPPORTED_ID_START+59)
-#define LOOK_DontAnimateBackground_ID			(LOOK_SUPPORTED_ID_START+60)
-#define LOOK_CoverAnimationSteps_ID				(LOOK_SUPPORTED_ID_START+61)				  
-#define LOOK_CoverAnimationType_ID				(LOOK_SUPPORTED_ID_START+62)
-#define LOOK_SupportedHints_ID  				(LOOK_SUPPORTED_ID_START+63)							  
+#define LOOK_WindowStyle_ID_END					(LOOK_WindowStyle_ID_START+3)
+#define LOOK_MenuStyle_ID_START					(LOOK_WindowStyle_ID_END+1)
+#define LOOK_MenuItemStyle_ID					(LOOK_MenuStyle_ID_START)					  
+#define LOOK_MenuTitleStyle_ID					(LOOK_MenuStyle_ID_START+1)					  
+#define LOOK_MenuHiliteStyle_ID					(LOOK_MenuStyle_ID_START+2)					  
+#define LOOK_MenuStippleStyle_ID				(LOOK_MenuStyle_ID_START+3)					  
+#define LOOK_MenuSubItemStyle_ID				(LOOK_MenuStyle_ID_START+4)					  
+#define LOOK_MenuHiTitleStyle_ID				(LOOK_MenuStyle_ID_START+6)
+#define LOOK_MenuStyle_ID_END					(LOOK_MenuStyle_ID_START+6)		  
+#define LOOK_MenuItemCompositionMethod_ID		(LOOK_MenuStyle_ID_END+1)	  
+#define LOOK_MenuHiliteCompositionMethod_ID		(LOOK_MenuStyle_ID_END+2)
+#define LOOK_MenuStippleCompositionMethod_ID	(LOOK_MenuStyle_ID_END+3)
+#define LOOK_ShadeAnimationSteps_ID				(LOOK_MenuStyle_ID_END+4)	  
+#define LOOK_TitleButtonBalloonBorderHilite_ID	(LOOK_MenuStyle_ID_END+5)
+#define LOOK_TitleButtonBalloonXOffset_ID		(LOOK_MenuStyle_ID_END+6)
+#define LOOK_TitleButtonBalloonYOffset_ID		(LOOK_MenuStyle_ID_END+7)
+#define LOOK_TitleButtonBalloonDelay_ID			(LOOK_MenuStyle_ID_END+8)		  
+#define LOOK_TitleButtonBalloonCloseDelay_ID	(LOOK_MenuStyle_ID_END+9)
+#define LOOK_TitleButtonBalloonStyle_ID			(LOOK_MenuStyle_ID_END+10)
+#define LOOK_TitleButtonBalloons_ID				(LOOK_MenuStyle_ID_END+11)
+#define LOOK_TitleButton_ID						(LOOK_MenuStyle_ID_END+12)
+#define LOOK_KillBackgroundThreshold_ID			(LOOK_MenuStyle_ID_END+13)
+#define LOOK_DontAnimateBackground_ID			(LOOK_MenuStyle_ID_END+14)
+#define LOOK_CoverAnimationSteps_ID				(LOOK_MenuStyle_ID_END+15)				  
+#define LOOK_CoverAnimationType_ID				(LOOK_MenuStyle_ID_END+16)
+#define LOOK_SupportedHints_ID  				(LOOK_MenuStyle_ID_END+17)							  
+#define LOOK_MinipixmapSize_ID					(LOOK_MenuStyle_ID_END+18)
 
-#define LOOK_SUPPORTED_ID_END		(LOOK_SUPPORTED_ID_START+64)
+#define LOOK_SUPPORTED_ID_END		(LOOK_MenuStyle_ID_END+32)
 #define LOOK_ID_END			(LOOK_SUPPORTED_ID_END)
 
 typedef struct DesktopConfig
@@ -1470,6 +1494,8 @@ void DestroyLookConfig (LookConfig * config);
 ASFlagType ParseBevelOptions( struct FreeStorageElem * options );
 ASFlagType ParseAlignOptions( struct FreeStorageElem * options );
 void bevel_parse(char *text, FILE * fd, char **myname, int *pbevel);
+void align_parse(char *text, FILE * fd, char **myname, int *palign);
+
 LookConfig *ParseLookOptions (const char *filename, char *myname);
 struct MyLook *LookConfig2MyLook ( struct LookConfig *config, struct MyLook *look,
 			  			   	       struct ASImageManager *imman,
@@ -1633,9 +1659,11 @@ struct FreeStorageElem **ComplexFunction2FreeStorage( struct SyntaxDef *syntax,
 #define WINDOWBOX_MaxLayer_ID           (WINDOWBOX_ID_START+11)
 #define WINDOWBOX_VerticalPriority_ID   (WINDOWBOX_ID_START+12)
 #define WINDOWBOX_ReverseOrder_ID       (WINDOWBOX_ID_START+13)
-#define WINDOWBOX_DONE_ID               (WINDOWBOX_ID_START+14)
+#define WINDOWBOX_ReverseOrderHorizontal_ID	(WINDOWBOX_ID_START+14)
+#define WINDOWBOX_ReverseOrderVertical_ID  	(WINDOWBOX_ID_START+15)
+#define WINDOWBOX_DONE_ID               (WINDOWBOX_ID_START+16)
 
-#define WINDOWBOX_ID_END            	(WINDOWBOX_ID_START+16)
+#define WINDOWBOX_ID_END            	(WINDOWBOX_ID_START+17)
 
 /* we don't really need FeelConfig since feel does not cary
    any information that requires char2bin conversion and consecutive
@@ -1848,6 +1876,7 @@ void LoadColorScheme();                        /* high level easy to use functio
 typedef struct WinTabsConfig
 {
 #define WINTABS_UseSkipList		(0x01<<1)
+#define WINTABS_HideWhenEmpty  	(0x01<<1)
 #define WINTABS_Geometry		(0x01<<2)
 #define WINTABS_MaxRows			(0x01<<5)
 #define WINTABS_MaxColumns		(0x01<<6)
@@ -1865,11 +1894,13 @@ typedef struct WinTabsConfig
 
 #define WINTABS_H_SPACING       (0x01<<18)
 #define WINTABS_V_SPACING       (0x01<<19)
+#define WINTABS_PatternType	    (0x01<<20)
 
 #define 	ASWT_UseSkipList	WINTABS_UseSkipList
+#define 	ASWT_HideWhenEmpty	WINTABS_HideWhenEmpty
 	ASFlagType	flags ;
 	ASFlagType	set_flags ;
-    ASGeometry geometry ;
+    ASGeometry  geometry ;
 #define MAX_WINTABS_WINDOW_COUNT    64
 	unsigned int max_rows, max_columns ;
     unsigned int min_tab_width, max_tab_width ;
