@@ -66,6 +66,7 @@
 
 static void decode_asscanline_native( ASImageDecoder *imdec, unsigned int skip, int y );
 static void decode_asscanline_ximage( ASImageDecoder *imdec, unsigned int skip, int y );
+static void decode_asscanline_argb32( ASImageDecoder *imdec, unsigned int skip, int y );
 
 void decode_image_scanline_normal( ASImageDecoder *imdec );
 void decode_image_scanline_beveled( ASImageDecoder *imdec );
@@ -195,7 +196,7 @@ asimage_erase_line( ASImage * im, ColorPart color, unsigned int y )
 
 
 /* for consistency sake : */
-inline void
+void
 copy_component( register CARD32 *src, register CARD32 *dst, int *unused, int len )
 {
 #if 1
@@ -315,11 +316,11 @@ start_image_decoding( ASVisual *asv,ASImage *im, ASFlagType filter,
 	}else
 	{
 		if( offset_x < 0 )
-			offset_x = im->width - (offset_x%im->width);
+			offset_x = (int)im->width + (offset_x%(int)im->width);
 		else
 			offset_x %= im->width ;
 		if( offset_y < 0 )
-			offset_y = im->height - (offset_y%im->height);
+			offset_y = (int)im->height + (offset_y%(int)im->height);
 		else
 			offset_y %= im->height ;
 		if( out_width == 0 )
@@ -381,15 +382,22 @@ start_image_decoding( ASVisual *asv,ASImage *im, ASFlagType filter,
 	prepare_scanline(out_width+imdec->bevel_h_addon, 0, &(imdec->buffer), asv->BGR_mode );
     imdec->buffer.back_color = (im != NULL)?im->back_color:ARGB32_DEFAULT_BACK_COLOR ;
 
-	if( im == NULL )
-		imdec->decode_asscanline = decode_asscanline_native;
-	else if( get_flags( im->flags, ASIM_DATA_NOT_USEFUL ) && im->alt.ximage != NULL )
-	{
-		imdec->decode_asscanline = decode_asscanline_ximage;
-		imdec->xim_buffer = safecalloc(1, sizeof(ASScanline));
-		prepare_scanline(im->alt.ximage->width, 0, imdec->xim_buffer, asv->BGR_mode );
-	}else
-		imdec->decode_asscanline = decode_asscanline_native;
+	imdec->decode_asscanline = decode_asscanline_native;
+	if( im != NULL )
+	{	
+		if( get_flags( im->flags, ASIM_DATA_NOT_USEFUL ) )
+		{
+			if( im->alt.ximage != NULL && !get_flags( im->flags, ASIM_XIMAGE_NOT_USEFUL) )
+			{
+				imdec->decode_asscanline = decode_asscanline_ximage;
+				imdec->xim_buffer = safecalloc(1, sizeof(ASScanline));
+				prepare_scanline(im->alt.ximage->width, 0, imdec->xim_buffer, asv->BGR_mode );
+			}else if( im->alt.argb32 != NULL )
+			{
+				imdec->decode_asscanline = decode_asscanline_argb32;
+			}	 
+		}
+	}
 
 	return imdec;
 }
@@ -736,7 +744,7 @@ decode_asscanline_native( ASImageDecoder *imdec, unsigned int skip, int y )
 		{
 			register CARD32 *chan = scl->channels[i]+skip;
 			if( imdec->im )
-				count = fetch_data32( NULL, imdec->im->channels[i][y], chan, imdec->offset_x, width, 0);
+				count = fetch_data32( NULL, imdec->im->channels[i][y], chan, imdec->offset_x, width, 0, NULL);
 			else
 				count = 0 ;
 			if( scl->shift )
@@ -752,6 +760,62 @@ decode_asscanline_native( ASImageDecoder *imdec, unsigned int skip, int y )
 	set_flags( scl->flags, imdec->filter);
 }
 
+static void
+decode_asscanline_argb32( ASImageDecoder *imdec, unsigned int skip, int y )
+{
+	ASScanline *scl = &(imdec->buffer);
+	int count, width = scl->width-skip ;
+	ARGB32 *row = imdec->im->alt.argb32 + y*imdec->im->width ;
+	CARD32 *a = scl->alpha+skip;
+	CARD32 *r = scl->red+skip;
+	CARD32 *g = scl->green+skip;
+	CARD32 *b = scl->blue+skip;
+	int max_x = imdec->im->width ;
+
+	if( get_flags( imdec->filter, SCL_DO_ALPHA ) )
+	{
+		int x = imdec->offset_x ; 
+		for( count = 0 ; count < width ; ++count) 
+		{	
+			a[count] = ARGB32_ALPHA8(row[x])<<scl->shift ;	
+			if( ++x >= max_x ) 	x = 0;
+		}	 
+	}	 
+		
+	if( get_flags( imdec->filter, SCL_DO_RED ) )
+	{
+		int x = imdec->offset_x ; 
+		for( count = 0 ; count < width ; ++count) 
+		{	
+			r[count] = ARGB32_RED8(row[x])<<scl->shift ;	
+			if( ++x >= max_x ) 	x = 0;
+		}	 
+	}	 
+		
+	if( get_flags( imdec->filter, SCL_DO_GREEN ) )
+	{
+		int x = imdec->offset_x ; 
+		for( count = 0 ; count < width ; ++count) 
+		{	
+			g[count] = ARGB32_GREEN8(row[x])<<scl->shift ;	
+			if( ++x >= max_x ) 	x = 0;
+		}	 
+	}	 
+	if( get_flags( imdec->filter, SCL_DO_BLUE ) )
+	{
+		int x = imdec->offset_x ; 
+		for( count = 0 ; count < width ; ++count) 
+		{	
+			b[count] = ARGB32_BLUE8(row[x])<<scl->shift ;	
+			if( ++x >= max_x ) 	x = 0;
+		}	 
+	}	 
+
+	clear_flags( scl->flags, SCL_DO_ALL);
+	set_flags( scl->flags, imdec->filter);
+}
+
+
 static void
 decode_asscanline_ximage( ASImageDecoder *imdec, unsigned int skip, int y )
 {
@@ -767,7 +831,7 @@ decode_asscanline_ximage( ASImageDecoder *imdec, unsigned int skip, int y )
 		int offset_x = imdec->offset_x%xim_width ;
 /*fprintf( stderr, __FILE__ ":" __FUNCTION__ ": width=%d, xim_width=%d, skip = %d, offset_x = %d - tiling\n", width, xim->width, skip, imdec->offset_x );	*/
 
-		GET_SCANLINE(imdec->asv,xim,xim_scl,y,xim->data+xim->bytes_per_line*y);
+		GET_SCANLINE(imdec->asv,xim,xim_scl,y,(unsigned char*)xim->data+xim->bytes_per_line*y);
 		/* We also need to decode mask if we have one :*/
 		if( (xim = imdec->im->alt.mask_ximage ) != NULL )
 		{
@@ -776,7 +840,7 @@ decode_asscanline_ximage( ASImageDecoder *imdec, unsigned int skip, int y )
 			register int x = MIN((int)xim_scl->width,xim->width);
 			if( xim->depth == 8 )
 			{
-				CARD8  *src = xim->data+xim->bytes_per_line*y ;
+				CARD8  *src = (CARD8*)xim->data+xim->bytes_per_line*y ;
 				while(--x >= 0 ) dst[x] = (CARD32)(src[x]);
 			}else
 			{
@@ -819,7 +883,7 @@ decode_asscanline_ximage( ASImageDecoder *imdec, unsigned int skip, int y )
 /*fprintf( stderr, __FILE__ ":" __FUNCTION__ ":direct\n" );	*/
 		int old_offset = scl->offset_x ;
 		scl->offset_x = skip ;
-		GET_SCANLINE(imdec->asv,xim,scl,y,xim->data+xim->bytes_per_line*y);
+		GET_SCANLINE(imdec->asv,xim,scl,y,(unsigned char *)xim->data+xim->bytes_per_line*y);
 		/* We also need to decode mask if we have one :*/
 		if( (xim = imdec->im->alt.mask_ximage ) != NULL )
 		{
@@ -828,7 +892,7 @@ decode_asscanline_ximage( ASImageDecoder *imdec, unsigned int skip, int y )
 			register int x = MIN(width,xim_width);
 			if( xim->depth == 8 )
 			{
-				CARD8  *src = xim->data+xim->bytes_per_line*y ;
+				CARD8  *src = (CARD8*)xim->data+xim->bytes_per_line*y ;
 				while(--x >= 0 ) dst[x] = (CARD32)(src[x]);
 			}else
 			{
@@ -1201,7 +1265,7 @@ encode_image_scanline_mask_xim( ASImageOutput *imout, ASScanline *to_store )
 			register int x = MIN((unsigned int)(xim->width), to_store->width);
 			if( xim->depth == 8 )
 			{
-				CARD8 *dst = xim->data+xim->bytes_per_line*imout->next_line ;
+				CARD8 *dst = (CARD8*)xim->data+xim->bytes_per_line*imout->next_line ;
 				while( --x >= 0 )
 					dst[x] = (CARD8)(a[x]);
 			}else
