@@ -184,7 +184,7 @@ AddWindow (Window w)
 	 */
 	XSelectInput( dpy, w, AS_CLIENT_EVENT_MASK );
 		
-    if( collect_hints( &Scr, w, HINT_ANY, &raw_hints ) )
+    if( collect_hints( ASDefaultScr, w, HINT_ANY, &raw_hints ) )
     {
         if( is_output_level_under_threshold(OUTPUT_LEVEL_HINTS) )
             print_hints( NULL, NULL, &raw_hints );
@@ -218,7 +218,6 @@ AddWindow (Window w)
     tmp_win->wm_state_transition = get_flags(status.flags, AS_Iconic)?ASWT_Withdrawn2Iconic:ASWT_Withdrawn2Normal ;
 
     pending_placement = init_aswindow_status( tmp_win, &status );
-
 #ifdef SHAPE
     XShapeSelectInput (dpy, tmp_win->w, ShapeNotifyMask);
 #endif
@@ -231,10 +230,8 @@ AddWindow (Window w)
 	 * gotten one for anything up to here, however.
 	 */
     ASSync( False );
-	if( !pending_placement )
-	{
-	    grab_server();
-	}
+    grab_server();
+/*vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv Start Grab vvvvvvvvvvvvvvvvvvvvv */
     if (validate_drawable(tmp_win->w, NULL, NULL) == None)
 	{
 		destroy_hints(tmp_win->hints, False);
@@ -247,23 +244,40 @@ AddWindow (Window w)
 	/* add the window into the afterstep list */
     enlist_aswindow( tmp_win );
     redecorate_window  ( tmp_win, False );
-    on_window_title_changed ( tmp_win, False );
-
-    if( pending_placement )
-    {
-		RaiseWindow(tmp_win);
-        on_window_status_changed( tmp_win, False, True );
-        if( !place_aswindow( tmp_win ) )
-        {
-            LOCAL_DEBUG_OUT( "window status initialization failed for %lX - destroying it", w );
-            Destroy (tmp_win, False);
-            return NULL;
-        }
-    }
     /* saving window management properties : */
     set_client_desktop( tmp_win->w, ASWIN_DESK(tmp_win) );
-    set_window_wm_state( tmp_win, get_flags(status.flags, AS_Iconic) );
+
+	/* we have to set shape on frame window. If window has title - 
+	 * on_window_title_changed will take care of it - otherwise we force it
+	 * by calling SetShape directly.
+	 */
+
+	if( tmp_win->tbar )
+	    on_window_title_changed ( tmp_win, False );
+	else 
+		SetShape( tmp_win, 0 );
+	/* Must do it now or else Java will freak out !!! */
+    XMapRaised (dpy, tmp_win->w);
 	RaiseWindow(tmp_win);
+
+/*^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ End Grab ^^^^^^^^^^^^^^^^^^^^ */
+	ungrab_server();
+
+	/* doing that for any window seems to resolve java sizing bugs */
+    on_window_status_changed( tmp_win, False, True );
+    if( pending_placement )
+    {
+        if( !place_aswindow( tmp_win ) )
+        {
+            LOCAL_DEBUG_OUT( "window status initialization failed for %lX - will put it where it is", w );
+            /* 	Destroy (tmp_win, False);
+            	return NULL;
+			 */
+        }
+    }
+
+
+    set_window_wm_state( tmp_win, get_flags(status.flags, AS_Iconic) );
  
 	if( ASWIN_HFLAGS( tmp_win, AS_AvoidCover )  )
 		enforce_avoid_cover( tmp_win );
@@ -274,8 +288,6 @@ AddWindow (Window w)
 	 * WithdrawnState in HandleUnmapNotify.  Map state gets set correctly
 	 * again in HandleMapNotify.
      */
-	if( !pending_placement )
-		ungrab_server();
     broadcast_config (M_ADD_WINDOW, tmp_win);
     broadcast_window_name( tmp_win );
     broadcast_res_names( tmp_win );
@@ -456,6 +468,7 @@ LOCAL_DEBUG_CALLER_OUT( "asw(%p)->internal(%p)->data(%p)", asw, asw->internal, a
 
     redecorate_window( asw, True );
     unregister_aswindow( asw->w );
+	remove_iconbox_icon( asw );
     delist_aswindow( asw );
 
     init_aswindow( asw, True );
